@@ -38,10 +38,13 @@ use Coro::State;
 
 use base Exporter;
 
-$VERSION = 0.13;
+$VERSION = 0.45;
 
 @EXPORT = qw(async cede schedule terminate current);
-@EXPORT_OK = qw($current);
+%EXPORT_TAGS = (
+      prio => [qw(PRIO_MAX PRIO_HIGH PRIO_NORMAL PRIO_LOW PRIO_IDLE PRIO_MIN)],
+);
+@EXPORT_OK = @{$EXPORT_TAGS{prio}};
 
 {
    my @async;
@@ -120,9 +123,6 @@ my $manager = new Coro sub {
    }
 };
 
-# we really need priorities...
-my @ready; # the ready queue. hehe, rather broken ;)
-
 # static methods. not really.
 
 =head2 STATIC METHODS
@@ -162,14 +162,6 @@ never be called again.
 
 =cut
 
-my $prev;
-
-sub schedule {
-   # should be done using priorities :(
-   ($prev, $current) = ($current, shift @ready || $idle);
-   Coro::State::transfer($prev, $current);
-}
-
 =item cede
 
 "Cede" to other processes. This function puts the current process into the
@@ -178,11 +170,6 @@ current "timeslice" to other coroutines of the same or higher priority.
 
 =cut
 
-sub cede {
-   $current->ready;
-   &schedule;
-}
-
 =item terminate
 
 Terminates the current process.
@@ -192,10 +179,9 @@ Future versions of this function will allow result arguments.
 =cut
 
 sub terminate {
-   push @destroy, $current;
-   $manager->ready;
+   $current->cancel;
    &schedule;
-   # NORETURN
+   die; # NORETURN
 }
 
 =back
@@ -236,8 +222,55 @@ Put the current process into the ready queue.
 
 =cut
 
-sub ready {
-   push @ready, $_[0];
+=item $process->cancel
+
+Like C<terminate>, but terminates the specified process instead.
+
+=cut
+
+sub cancel {
+   push @destroy, $_[0];
+   $manager->ready;
+}
+
+=item $oldprio = $process->prio($newprio)
+
+Sets the priority of the process. Higher priority processes get run before
+lower priority processes. Priorities are smalled signed integer (currently
+-4 .. +3), that you can refer to using PRIO_xxx constants (use the import
+tag :prio to get then):
+
+   PRIO_MAX > PRIO_HIGH > PRIO_NORMAL > PRIO_LOW > PRIO_IDLE > PRIO_MIN
+       3    >     1     >      0      >    -1    >    -3     >    -4
+
+   # set priority to HIGH
+   current->prio(PRIO_HIGH);
+
+The idle coroutine ($Coro::idle) always has a lower priority than any
+existing coroutine.
+
+Changing the priority of the current process will take effect immediately,
+but changing the priority of processes in the ready queue (but not
+running) will only take effect after the next schedule (of that
+process). This is a bug that will be fixed in some future version.
+
+=cut
+
+sub prio {
+   my $old = $_[0]{prio};
+   $_[0]{prio} = $_[1] if @_ > 1;
+   $old;
+}
+
+=item $newprio = $process->nice($change)
+
+Similar to C<prio>, but subtract the given value from the priority (i.e.
+higher values mean lower priority, just as in unix).
+
+=cut
+
+sub nice {
+   $_[0]{prio} -= $_[1];
 }
 
 =back
